#ifndef COMMANDQUEUE_H
#define COMMANDQUEUE_H

#include "command.h"

typedef struct
 {
  Command *start, *end, *alloc;
 } CommandQueue;

int commandQueueInit(CommandQueue *);
void commandQueueUninit(CommandQueue *);

uint16_t commandQueueElements(const CommandQueue *);

/* Returned pointer won't be valid after pushing or popping. . . */
const Command * commandQueueElement(const CommandQueue *, uint16_t i);

/* Note of caution: When you push a command, that command's data ends up being used so you must
   not use it or uninit it.
   The pointer you pass isn't used, however. If you allocated it, feel free to free it after you
   push the command. If pushing fails, though, then it's your job to uninit the command first. */

int commandQueueForgetUser(CommandQueue *q, uint8_t user);
int commandQueuePushFront(CommandQueue *q, Command *);
int commandQueuePushBack(CommandQueue *q, Command *);
int commandQueuePopFront(CommandQueue *q, Command *);
int commandQueuePopBack(CommandQueue *q, Command *);

int commandQueueInitFromFile(CommandQueue *, bzf *file);
int commandQueueSave(CommandQueue *, bzf *file);
int commandQueueInitFilename(CommandQueue *q, const char *filename);
int commandQueueSaveFilename(CommandQueue *q, const char *filename);

#endif
