#include <stdio.h>
#include "SDL/SDL.h"
#include <GL/gl.h>

#include "glcanvas.h"

static Canvas *canvas = 0;

static void uninit()
 {
  if(canvas)
   {
    destroyGLCanvas(canvas);
    canvas = 0;
   }
  
  SDL_Quit();
 }

static void screenReshaped(int w, int h)
 {
  glEnable(GL_POINT_SMOOTH);
  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
  
  glViewport(0, 0, w, h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, w, h, 0, -1, 1);
  
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
 }

int main(int argc, char *argv[])
 {
  SDL_Surface *screen;
  
  if(SDL_Init(SDL_INIT_VIDEO) < 0)
   {
    fprintf(stderr, "SDL_Init failed: %s\n", SDL_GetError());
    return 1;
   }
  
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
  SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 1);
  
  if(!(screen=SDL_SetVideoMode(800, 600, 32, SDL_OPENGL | SDL_RESIZABLE)))
   {
    fprintf(stderr, "SDL_SetVideoMode failed: %s\n", SDL_GetError());
    SDL_Quit();
    return 1;
   }
  
  screenReshaped(screen->w, screen->h);
  
  canvas = createGLCanvas(1024, 768);
  renderGLCanvasMandelbrot(canvas);
  
  SDL_Event event;
  
  CanvasPoint p1 = {0.0, 0.0};
  float remainder = 0.0;
  
  while(1)
   {
    while(SDL_PollEvent(&event)) switch(event.type)
     {
      case SDL_VIDEORESIZE:
       {
        int w = screen->w, h = screen->h;
        Uint8 bpp = screen->format->BitsPerPixel;
        Uint32 f = screen->flags;
        
        if(!(screen=SDL_SetVideoMode(event.resize.w, event.resize.h, bpp, f)) &&
           !(screen=SDL_SetVideoMode(w, h, bpp, f)))
         {
          fprintf(stderr, "SDL_SetVideoMode failed: %s\n", SDL_GetError());
          uninit(); // Going to abort, lost our video and can't get it back.
          return 1;
         }
        
        screenReshaped(screen->w, screen->h);
       }
      break;
      
      case SDL_KEYDOWN:
       if(event.key.keysym.sym == SDLK_F11)
        {
         int w = screen->w, h = screen->h;
         Uint8 bpp = screen->format->BitsPerPixel;
         Uint32 f = screen->flags, new_f = f^SDL_FULLSCREEN;
         
         if(!(screen=SDL_SetVideoMode(w, h, bpp, new_f)) &&
            !(screen=SDL_SetVideoMode(800, 600, bpp, new_f)) &&
            !(screen=SDL_SetVideoMode(w, h, bpp, f)))
          {
           fprintf(stderr, "SDL_SetVideoMode failed: %s\n", SDL_GetError());
           uninit(); // Going to abort, lost our video and can't get it back.
           return 1;
          }
         
         screenReshaped(screen->w, screen->h);
        }
      break;
      
      case SDL_MOUSEMOTION:
       {
        CanvasPoint p2 = { event.motion.x, event.motion.y };
        
        pushCanvasLine(canvas, p1, p2, &remainder);
        p1 = p2;
        
        dumpCanvasPoints(canvas, 0, 0, 0, 255);
       }
      break;
      
      case SDL_QUIT:
       uninit();
      return 0;
     }
    
    glClear(GL_COLOR_BUFFER_BIT);
    
    drawGLCanvas(canvas);
    
    SDL_GL_SwapBuffers();
   }
 }
