#include "tools.h"
#include "draw.h"
#include "rng.h"

/* from client.c */
extern Colour colour;
extern int16_t y[];
extern int16_t x[];
extern uint32_t points;

extern BITMAP *preview;

int preview_points = 4;
int16_t preview_x[] = {20, 140, 20, 140},
        preview_y[] = {20, 40,  80, 100};

void drawRadiusBar(BITMAP *map, int y, int w)
 {
  int x = 0;
  for(; x < w; ++x)
   {
    int c = 255-255*(((float)x*32.0/(float)w)-(int)(x*32/w));
    vline(map, x, y, y+31-x*32/w, makecol(255, 255, 255));
    vline(map, x, y+31-x*32/w, y+31, makecol(0, 0, 0));
    _putpixel32(map, x, y+31-x*32/w, makecol(c, c, c));
   }
 }

void drawRateBar(BITMAP *map, int y, int w)
 {
  int x = 0;
  for(; x < w; ++x)
   vline(map, x, y, y+31, makecol(x*255/w, x*255/w, x*255/w));
 }

void drawVerticalMarker(BITMAP *map, int x, int y)
 {
  vline(map, x-2, y-10, y+10, makecol32(255, 0, 0));
  vline(map, x+2, y-10, y+10, makecol32(255, 0, 0));
  vline(map, x-3, y-9, y+9, makecol32(0, 0, 0));
  vline(map, x+3, y-9, y+9, makecol32(0, 0, 0));
 }

void drawHorizontalMarker(BITMAP *map, int x, int y)
 {
  hline(map, x-10, y-2, x+10, makecol32(255, 0, 0));
  hline(map, x-10, y+2, x+10, makecol32(255, 0, 0));
  hline(map, x-9, y-3, x+9, makecol32(0, 0, 0));
  hline(map, x-9, y+3, x+9, makecol32(0, 0, 0));
 }

void drawCrossMarker(BITMAP *map, int x, int y)
 {
  hline(map, x-2, y-2, x-10, makecol32(255, 0, 0));
  hline(map, x+2, y+2, x+10, makecol32(255, 0, 0));
  
  vline(map, x-2, y-3, y-10, makecol32(255, 0, 0));
  vline(map, x+2, y+3, y+10, makecol32(255, 0, 0));
  
  hline(map, x-3, y-3, x-9, makecol32(0, 0, 0));
  hline(map, x+3, y+3, x+9, makecol32(0, 0, 0));
  
  vline(map, x-3, y-4, y-9, makecol32(0, 0, 0));
  vline(map, x+3, y+4, y+9, makecol32(0, 0, 0));
  
  /*
  hline(map, x-10, y+2, x+10, makecol32(255, 0, 0));
  hline(map, x-9, y-3, x+9, makecol32(0, 0, 0));
  hline(map, x-9, y+3, x+9, makecol32(0, 0, 0)); */
 }

int penGenerate(Command *command)
 {
  return commandPenInit(command, colour, x, y, points);
 }

float pencil_radius = 1.7;

BITMAP * pencilOptions(BITMAP *bmp, int btn, int key, int x, int y)
 {
  int redraw = 0;
  
  if(!bmp)
   {
    if(!(bmp = create_bitmap_ex(32, preview->w, preview->h+32)))
     return 0;
    
    redraw = 1;
   }
  
  if(btn == 1 && y >= preview->h)
   {
    pencil_radius = pencil_min_radius + x/(float)preview->w*(pencil_max_radius - pencil_min_radius);
    redraw = 1;
   }
  
  if(redraw)
   {
    blit(preview, bmp, 0, 0, 0, 0, preview->w, preview->h);
    RNGPush(1);
    drawPencil(bmp, colour, pencil_radius, preview_x, preview_y, preview_points);
    RNGPop();
    drawRadiusBar(bmp, preview->h, preview->w);
    drawVerticalMarker(bmp, (pencil_radius-pencil_min_radius)/(pencil_max_radius-pencil_min_radius)*preview->w, preview->h+16);
    return bmp;
   }
  
  return 0;
 }

int pencilGenerate(Command *command)
 {
  return commandPencilInit(command, colour, pencil_radius, x, y, points);
 }

float brush_radius = 3.5, brush_rate = 100.0;

BITMAP * brushOptions(BITMAP *bmp, int btn, int key, int x, int y)
 {
  int redraw = 0;
  
  if(!bmp)
   {
    if(!(bmp = create_bitmap_ex(32, preview->w, preview->h+32*2)))
     return 0;
    
    redraw = 1;
   }
  
  if(btn == 1 && y >= preview->h)
   {
    if(y < preview->h+32)
     brush_radius = brush_min_radius + x/(float)preview->w*(brush_max_radius - brush_min_radius);
    else
     brush_rate = brush_min_rate + x/(float)preview->w*(brush_max_rate - brush_min_rate);
    
    redraw = 1;
   }
  
  if(redraw)
   {
    blit(preview, bmp, 0, 0, 0, 0, preview->w, preview->h);
    RNGPush(1);
    drawBrush(bmp, colour, brush_radius, brush_rate, preview_x, preview_y, preview_points);
    RNGPop();
    drawRadiusBar(bmp, preview->h, preview->w);
    drawRateBar(bmp, preview->h+32, preview->w);
    drawVerticalMarker(bmp, (brush_radius-brush_min_radius)/(brush_max_radius-brush_min_radius)*preview->w, preview->h+16);
    drawVerticalMarker(bmp, (brush_rate-brush_min_rate)/(brush_max_rate-brush_min_rate)*preview->w, preview->h+32+16);
    return bmp;
   }
  
  return 0;
 }

int brushGenerate(Command *command)
 {
  return commandBrushInit(command, colour, brush_radius, brush_rate, x, y, points);
 }

float chalk_radius = 5.5;

BITMAP * chalkOptions(BITMAP *bmp, int btn, int key, int x, int y)
 {
  int redraw = 0;
  
  if(!bmp)
   {
    if(!(bmp = create_bitmap_ex(32, preview->w, preview->h+32)))
     return 0;
    
    redraw = 1;
   }
  
  if(btn == 1 && y >= preview->h)
   {
    chalk_radius = chalk_min_radius + x/(float)preview->w*(chalk_max_radius - chalk_min_radius);
    redraw = 1;
   }
  
  if(redraw)
   {
    blit(preview, bmp, 0, 0, 0, 0, preview->w, preview->h);
    RNGPush(1);
    drawChalk(bmp, colour, chalk_radius, preview_x, preview_y, preview_points);
    RNGPop();
    drawRadiusBar(bmp, preview->h, preview->w);
    drawVerticalMarker(bmp, (chalk_radius-chalk_min_radius)/(chalk_max_radius-chalk_min_radius)*preview->w, preview->h+16);
    return bmp;
   }
  
  return 0;
 }

int chalkGenerate(Command *command)
 {
  return commandChalkInit(command, colour, chalk_radius, x, y, points);
 }

float smudge_radius = 8.5, smudge_rate = 600.0;

BITMAP * smudgeOptions(BITMAP *bmp, int btn, int key, int x, int y)
 {
  int redraw = 0;
  
  if(!bmp)
   {
    if(!(bmp = create_bitmap_ex(32, preview->w, preview->h+32*2)))
     return 0;
    
    redraw = 1;
   }
  
  if(btn == 1 && y >= preview->h)
   {
    if(y < preview->h+32)
     smudge_radius = smudge_min_radius + x/(float)preview->w*(smudge_max_radius - smudge_min_radius);
    else
     smudge_rate = smudge_min_rate + x/(float)preview->w*(smudge_max_rate - smudge_min_rate);
    
    redraw = 1;
   }
  
  if(redraw)
   {
    blit(preview, bmp, 0, 0, 0, 0, preview->w, preview->h);
    drawSmudge(bmp, smudge_radius, smudge_rate, preview_x, preview_y, preview_points);
    drawRadiusBar(bmp, preview->h, preview->w);
    drawRateBar(bmp, preview->h+32, preview->w);
    drawVerticalMarker(bmp, (smudge_radius-smudge_min_radius)/(smudge_max_radius-smudge_min_radius)*preview->w, preview->h+16);
    drawVerticalMarker(bmp, (smudge_rate-smudge_min_rate)/(smudge_max_rate-smudge_min_rate)*preview->w, preview->h+32+16);
    return bmp;
   }
  
  return 0;
 }

int smudgeGenerate(Command *command)
 {
  return commandSmudgeInit(command, smudge_radius, smudge_rate, x, y, points);
 }

float stamp_radius = 3.5, stamp_rate = 50.0;

BITMAP * stampOptions(BITMAP *bmp, int btn, int key, int x, int y)
 {
  int redraw = 0;
  
  if(!bmp)
   {
    if(!(bmp = create_bitmap_ex(32, preview->w, preview->h+32*2)))
     return 0;
    
    redraw = 1;
   }
  
  if(btn == 1 && y >= preview->h)
   {
    if(y < preview->h+32)
     stamp_radius = stamp_min_radius + x/(float)preview->w*(stamp_max_radius - stamp_min_radius);
    else
     stamp_rate = stamp_min_rate + x/(float)preview->w*(stamp_max_rate - stamp_min_rate);
    
    redraw = 1;
   }
  
  if(redraw)
   {
    blit(preview, bmp, 0, 0, 0, 0, preview->w, preview->h);
    drawStamp(bmp, colour, stamp_radius, stamp_rate, preview_x, preview_y, preview_points);
    drawRadiusBar(bmp, preview->h, preview->w);
    drawRateBar(bmp, preview->h+32, preview->w);
    drawVerticalMarker(bmp, (stamp_radius-stamp_min_radius)/(stamp_max_radius-stamp_min_radius)*preview->w, preview->h+16);
    drawVerticalMarker(bmp, (stamp_rate-stamp_min_rate)/(stamp_max_rate-stamp_min_rate)*preview->w, preview->h+32+16);
    return bmp;
   }
  
  return 0;
 }

int stampGenerate(Command *command)
 {
  return commandStampInit(command, colour, stamp_radius, stamp_rate, x, y, points);
 }
